@@ -120,6 +120,14 @@ class MenuEditor:
 			open(new_path, 'w').write(data)
 			os.unlink(file_path)
 			redo.append(redo_path)
+		#reload DOM to make changes stick
+		for name in ('applications', 'settings'):
+			menu = getattr(self, name)
+			if not os.path.isfile(menu.path):
+				menu.dom = xml.dom.minidom.parseString(util.getUserMenuXml(menu.tree))
+			else:
+				menu.dom = xml.dom.minidom.parse(menu.path)
+			self.__remove_whilespace_nodes(menu.dom)
 		self.__redo.append(redo)
 
 	def redo(self):
@@ -136,6 +144,14 @@ class MenuEditor:
 			open(new_path, 'w').write(data)
 			os.unlink(file_path)
 			undo.append(undo_path)
+		#reload DOM to make changes stick
+		for name in ('applications', 'settings'):
+			menu = getattr(self, name)
+			if not os.path.isfile(menu.path):
+				menu.dom = xml.dom.minidom.parseString(util.getUserMenuXml(menu.tree))
+			else:
+				menu.dom = xml.dom.minidom.parse(menu.path)
+			self.__remove_whilespace_nodes(menu.dom)
 		self.__undo.append(undo)
 
 	def getMenus(self, parent=None):
@@ -211,6 +227,7 @@ class MenuEditor:
 		file_id = self.__writeMenu(None, icon, name, comment)
 		menu_id = file_id.rsplit('.', 1)[0]
 		dom = self.__getMenu(parent).dom
+		self.__addXmlDefaultLayout(self.__getXmlMenu(self.__getPath(parent), dom, dom) , dom)
 		menu_xml = self.__getXmlMenu(self.__getPath(parent) + '/' + menu_id, dom, dom)
 		self.__addXmlTextElement(menu_xml, 'Directory', file_id, dom)
 		self.__positionItem(parent, ('Menu', menu_id), before, after)
@@ -427,10 +444,7 @@ class MenuEditor:
 
 	def __getPath(self, menu, path=None):
 		if not path:
-			if self.__getMenu(menu) == self.applications:
-				path = 'Applications'
-			else:
-				path = 'Desktop'
+                        path = menu.tree.root.get_menu_id()
 		if menu.get_parent():
 			path = self.__getPath(menu.get_parent(), path)
 			path += '/'
@@ -470,6 +484,10 @@ class MenuEditor:
 		return element.appendChild(node)
 
 	def __addXmlTextElement(self, element, name, text, dom):
+		for temp in element.childNodes:
+			if temp.nodeName == name:
+				if temp.childNodes[0].nodeValue == text:
+					return
 		node = dom.createElement(name)
 		text = dom.createTextNode(text)
 		node.appendChild(text)
@@ -496,7 +514,7 @@ class MenuEditor:
 			file_id = item.get_desktop_file_id()
 			keyfile = util.DesktopParser(file_path)
 		elif item == None and name == None:
-			raise Exception('New menu items need a name')
+			raise Exception(_('New menu items need a name'))
 		else:
 			file_id = util.getUniqueFileId(name, '.desktop')
 			keyfile = util.DesktopParser()
@@ -529,7 +547,7 @@ class MenuEditor:
 			file_path = menu.get_desktop_file_path()
 			keyfile = util.DesktopParser(file_path)
 		elif menu == None and name == None:
-			raise Exception('New menus need a name')
+			raise Exception(_('New menus need a name'))
 		else:
 			file_id = util.getUniqueFileId(name, '.directory')
 			keyfile = util.DesktopParser(file_type='Directory')
@@ -597,6 +615,16 @@ class MenuEditor:
 				node.appendChild(child)
 		return element.appendChild(node)
 
+	def __addXmlDefaultLayout(self, element, dom):
+		# remove old default layout
+		for node in self.__getXmlNodesByName('DefaultLayout', element):
+			element.removeChild(node)
+
+		# add new layout
+		node = dom.createElement('DefaultLayout')
+		node.setAttribute('inline', 'false')
+		return element.appendChild(node)
+
 	def __createLayout(self, items):
 		layout = Layout()
 		layout.order = []
